<?php
//
// Copyright (c) 2008-2009 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: dbl.inc.php 92 2010-04-15 19:23:01Z egalstad $

require_once(dirname(__FILE__).'/db.inc.php');


////////////////////////////////////////////////////////////////////////
// SQL GENERATION FUNCTIONS
////////////////////////////////////////////////////////////////////////
	

function generate_sql_query($dbh,$args){
	global $request;
	global $cfg;
	
	$dbtype='';
	if(array_key_exists("dbtype",$cfg['db_info'][$dbh]))
		$dbtype=$cfg['db_info'][$dbh]['dbtype'];
		
	// bail out if no sql
	if(!isset($args["sql"]))
		return "";

	$newsql=$args["sql"];

	// subsystem calls without a user id don't get limited
	$authlimit=true;
	if(defined("SUBSYSTEM_CALL")==true && !isset($args["uid"]))
		$authlimit=false;

	
	// default to using request args if none specified
	if(!isset($args["useropts"]))
		$args["useropts"]=$request;
		
	if(isset($request['debugsql']))
		echo "SQL[RAW]=$newsql  <BR>\n";

	// limit query (user specified args)
	$newsql=limit_sql_query($args,$dbtype);
	$args["sql"]=$newsql;
	if(isset($request['debugsql']))
		echo "SQL[LIM]=$newsql  <BR>\n";
		
	// limit to single instance id
	/*
	$newsql=limit_sql_to_instance($args,$dbtype);
	$args["sql"]=$newsql;
	if(isset($request['debugsql']))
		echo "SQL[INSTANCE]=$newsql  <BR>\n";
	*/
		
	// limit query by user authorization
	/*
	if($authlimit==true){
		$newsql=limit_sql_by_authorized_instance_ids($args,$dbtype);
		$args["sql"]=$newsql;
		if(isset($request['debugsql']))
			echo "SQL[AUTH-INSTANCE]=$newsql  <BR>\n";
		$newsql=limit_sql_by_authorized_object_ids($args,$dbtype);
		$args["sql"]=$newsql;
		if(isset($request['debugsql']))
			echo "SQL[AUTH-OBJECT]=$newsql  <BR>\n";
		}
	*/
		
	// optional group by statement
	if(isset($args["groupby"])){
		$newsql.=$args["groupby"];
		$args["sql"]=$newsql;
		if(isset($request['debugsql']))
			echo "SQL[GROUPBY]=$newsql  <BR>\n";
		}
	
	// order sql query
	$newsql=order_sql_query($args,$dbtype);
	$args["sql"]=$newsql;
	if(isset($request['debugsql']))
		echo "SQL[ORD]=$newsql  <BR>\n";

	// limit number of returned records
	if(!isset($args["limitrecords"]) || $args["limitrecords"]!=false){
		$newsql=limit_sql_query_records($args,$dbtype);
		$args["sql"]=$newsql;
		}


	return $newsql;
	}


function limit_sql_query($args,$dbtype){
	global $request;
	
	// bail out if no sql
	if(!isset($args["sql"]))
		return "";

	$newsql=$args["sql"];
	
	// if no field names were passed, don't limit query
	if(!isset($args["fieldmap"]))
		return $newsql;
	
	// otherwise, allow limiting...
	
	// default to using request if no args specified
	if(!isset($args["useropts"]))
		$args["useropts"]=$request;
	
	foreach($args["useropts"] as $var => $val){
	
		if(is_array($val)){
			foreach($val as $array_index => $array_val){
				$useropt=$var."=".$array_val;
				$newsql.=get_limited_arg_match_sql($useropt,$args["fieldmap"],$dbtype);
				}
			}
		else{
			$useropt=$var."=".$val;
			$newsql.=get_limited_arg_match_sql($useropt,$args["fieldmap"],$dbtype);
			}
		}
	
	return $newsql;
	}

function get_limited_arg_match_sql($useropt,$fieldmap,$dbtype){

	$newsql="";
	
	// split top-level args separated by semicolon - these are used to comprise OR statements
	// for instance:
	//      name=lk:d ; host_name=lkm:d,lks:d,in:3,4,5
	$topargs=explode(";",$useropt);
		
	// if we have multiple top-level args, we will do OR matches
	if(count($topargs)>1)
		$bool_type=" OR ";
	else
		$bool_type=" AND ";
			
	$current_toparg=0;
	foreach($topargs as $toparg){
		
		// we now have just one top-level arg
		// for instance:
		//      host_name=lkm:d,lks:d,in:3,4,5
			
		// split arg into fieldname and matchargs
		$topargsplit=explode("=",$toparg);
		$fieldname=current($topargsplit);
		$matchargs=next($topargsplit);
			
		// make sure fieldname is valid
		if(!array_key_exists($fieldname,$fieldmap))
			continue;

		// get translated table field name
		$table_fieldname=$fieldmap[$fieldname];
			
		$current_toparg++;

		if($current_toparg==1){
			$newsql.=" AND ";
			// enclose OR statements in parens
			if(count($topargs)>1)
				$newsql.="(";
			}

		if($current_toparg>1)
			$newsql.=$bool_type;
			
		//$newsql.="[$fieldname=$matchargs]";
		$newsql.=get_limited_field_match_sql($table_fieldname,$matchargs,$bool_type,$dbtype);
		}

	// close parens for OR statements
	if($current_toparg>=1 && count($topargs)>1)
		$newsql.=")";
		
	return $newsql;
	}

	
function get_limited_field_match_sql($table_fieldname,$matchargs,$bool_string,$dbtype){

	$newsql="";

	$found_option=false;
	$in_mid=false;
	$matchtype="eq";

	$args=explode(",",$matchargs);
			
	$current_arg=0;
	foreach($args as $arg){
	
		$current_arg++;
		  
		$parts=explode(":",$arg);
		
		if(count($parts)==1)
			$val=current($parts);
		else{
			$matchtype=current($parts);
			$val=next($parts);
			}
	
		// close previous "in" matches
		if($in_mid==true && ($matchtype!="in" && $matchtype!="nin")){
			$newsql.=")";
			$in_mid=false;
			}
			
		if($current_arg>1 && $in_mid==false)
			$newsql.=$bool_string;
					
		// what type of match should we perform?
		switch($matchtype){
	
			// not equal
			case "ne":
				$newsql.=$table_fieldname." != ".escape_sql_param($val,$dbtype,true);
				break;

			// less than
			case "lt":
				$newsql.=$table_fieldname." < ".escape_sql_param($val,$dbtype,true);
				break;
	
			// less than or equal
			case "lte":
				$newsql.=$table_fieldname." <= ".escape_sql_param($val,$dbtype,true);
				break;
	
			// greater than
			case "gt":
				$newsql.=$table_fieldname." > ".escape_sql_param($val,$dbtype,true);
				break;
	
			// greater than or equal
			case "gte":
				$newsql.=$table_fieldname." >= ".escape_sql_param($val,$dbtype,true);
				break;
	
			// beginning string match
			case "lks":
				$newsql.=$table_fieldname." LIKE '".escape_sql_param($val,$dbtype,false)."%'";
				break;
	
			// negative beginning string match
			case "nlks":
				$newsql.=$table_fieldname." NOT LIKE '".escape_sql_param($val,$dbtype,false)."%'";
				break;
	
			// ending string match
			case "lke":
				$newsql.=$table_fieldname." LIKE '%".escape_sql_param($val,$dbtype,false)."'";
				break;
	
			// negative ending string match
			case "nlke":
				$newsql.=$table_fieldname." NOT LIKE '%".escape_sql_param($val,$dbtype,false)."'";
				break;
	
			// mid string match
			case "lk":
			case "lkm":
				$newsql.=$table_fieldname." LIKE '%".escape_sql_param($val,$dbtype,false)."%'";
				break;
	
			// negative mid string match
			case "nlk":
			case "nlkm":
				$newsql.=$table_fieldname." NOT LIKE '%".escape_sql_param($val,$dbtype,false)."%'";
				break;
	
			// "in" match
			case "in":
				if($in_mid==false)
					$newsql.=$table_fieldname." IN (".escape_sql_param($val,$dbtype,true);
				else
					$newsql.=",".escape_sql_param($val,$dbtype,true);
				$in_mid=true;
				break;

			// negative "in" match
			case "nin":
				if($in_mid==false)
					$newsql.=$table_fieldname." NOT IN (".escape_sql_param($val,$dbtype,true);
				else
					$newsql.=",".escape_sql_param($val,true);
				$in_mid=true;
				break;

			// defaults to "equal" match type
			default:
				// mod 11/16/09 
				// some host/service names can contain colons, so we have to have an exception here if there was no specific "eq" specified
				if($matchtype!="eq"){
					$val=$matchtype.":".$val;
					}
				$newsql.=$table_fieldname." = ".escape_sql_param($val,$dbtype,true);
				break;
			}
		}

	// close "in" matches
	if($in_mid==true){
		$newsql.=")";
		$in_mid=false;
		}
			
	return $newsql;
	}


	
function order_sql_query($args,$dbtype){
	global $request;
	
	// bail out if no sql
	if(!isset($args["sql"]))
		return "";

	$newsql=$args["sql"];

	$foundoption=false;

	// if no field names were passed, we don't specify order
	if(!isset($args["fieldmap"]))
		return $newsql;

	// otherwise order the query...

	// default to using request if no args specified
	if(!isset($args["useropts"]))
		$args["useropts"]=$request;
	
	if(isset($args["useropts"]["orderby"]))
		$orderby=$args["useropts"]["orderby"];
	else
		$orderby=null;

	// no order specified in request, so order by first option desc (if not overridden)
	if(is_null($orderby)){
		if(!isset($args["default_order"])){
			reset($args["fieldmap"]);
		  	$newsql.=" ORDER BY ".current($args["fieldmap"])." DESC";
			}
		else if(have_value($args["default_order"]))
		  	$newsql.=" ORDER BY ".$args["default_order"];
		return $newsql;
		}

	// normal ordering...
  	$valarr=explode(",",$orderby);

  	foreach($valarr as $val){

		$parts=explode(":",$val);

		$userfieldname=current($parts);

		// make sure the field name is valid
		if(isset($args["fieldmap"][$userfieldname]))
			$fieldname=$args["fieldmap"][$userfieldname];
		else
			continue;
			
		if(count($parts)==1)
			$optorder="d";
		else
			$optorder=next($parts);

		$newsql.=($foundoption==true)?", ":" ORDER BY ";
		$newsql.=escape_sql_param($fieldname,$dbtype)." ";
		$newsql.=($optorder=="d" || $optorder=="desc")?"DESC":"ASC";

		$foundoption=true;
		}

	return $newsql;
	}
	

function limit_sql_query_records($args,$dbtype){
	global $request;
	global $cfg;

	// bail out if no sql
	if(!isset($args["sql"]))
		return "";
		
	$newsql=$args["sql"];

	// default to using request if no args specified
	if(!isset($args["useropts"]))
		$args["useropts"]=$request;
	
	$first_record=0;

	if(!isset($args["useropts"]['records']))
		$max_records=$cfg['default_result_records'];
	else{
		$parts=explode(":",$args["useropts"]['records']);
		$max_records=$parts[0];
		if(count($parts)>1)
			$first_record=$parts[1];
		}

	// sanitize
	$max_records=intval($max_records);
	$first_record=intval($first_record);

	// postgresql
	if($dbtype=='pgsql'){
		$newsql.=" LIMIT ";
		$newsql.=escape_sql_param($max_records,$dbtype);
		if($first_record>0)
			$newsql.=" OFFSET ".escape_sql_param($first_record,$dbtype);
		}
	// assume mysql-like syntax
	else{
		$newsql.=" LIMIT ";
		if($first_record>0)
			$newsql.=escape_sql_param($first_record,$dbtype).",";
		$newsql.=escape_sql_param($max_records,$dbtype);
		}

	if(isset($request['debugsql']))
		echo "SQL[CAP]=$newsql <BR>\n\n";

	return $newsql;
	}
	
?>