<?php
//
// Copyright (c) 2008-2009 Nagios Enterprises, LLC.  All rights reserved.
//
// Development Started 03/22/2008
// $Id: utils-systat.inc.php 18 2010-07-08 20:21:34Z egalstad $

//require_once(dirname(__FILE__).'/common.inc.php');



////////////////////////////////////////////////////////////////////////
// MISC SERVER FUNCTIONS
////////////////////////////////////////////////////////////////////////

function get_server_types(){
	$types=array(
		"nagiosxi" => "Nagios XI",
		"nagioscore" => "Nagios Core",
		"nagiosvshell" => "Nagios Core (V-Shell)", //added V-Shell Support -MG 4/07/11 
		);
	return $types;
	}
	
function get_server_type_from_short_name($name){

	$type=get_server_types();
	foreach($type as $sname => $lname){
		if($sname==$name)
			return $lname;
		}
	return "Unknown";
	}

function get_server_authentication_methods(){
	$types=array(
		"session" => "Session",
		"basic" => "HTTP Basic",
		);
	return $types;
	}

	
function get_server_root_url($sid,$frontend=true){

	// get server info
	$sinfo=get_server_info($sid);
	
	// get url
	if($frontend==true)
		$url=grab_array_var($sinfo,"url");
	else{
		$url=grab_array_var($sinfo,"internal_url");
		if($url=="")
			$url=grab_array_var($sinfo,"url");
		}
	
	// get server type
	$type=grab_array_var($sinfo,"type");
	
	// adjust url based on server type
	// TODO...
	
	return $url;
	}
	

////////////////////////////////////////////////////////////////////////
// SERVER STATE FUNCTIONS
////////////////////////////////////////////////////////////////////////

function get_server_state($sid,&$res,&$output){

	$result=false;
	$output="";

	$sinfo=get_server_info($sid);
	
	//print_r($sinfo);
	//echo "<BR><BR>";
	
	// check server
	$args=array(
		"timeout" => 5,
		"return_info" => true,
		);
	$url=get_server_root_url($sid,false);
	//echo "FETCHING $url<BR><BR>";
	$urlres=load_url($url,$args);
	
	$http_code=$urlres['info']['http_code'];

	if($http_code==0){
		$output="Unresponsive";
		$result=false;
		}
	else{
		$result=true;
		$output.="OK";   //removed http_code, no auth here -MG 5/3/11
		}
	
	//print_r($urlres);
	//echo "<BR><BR>";
	
	
	$res=$result;
	return $result;
	}
	
	
function check_server_auth($sid,$username,$password,&$res,&$output){

	$result=false;
	$output="";

	$sinfo=get_server_info($sid);
	
	//print_r($sinfo);
	//echo "<BR><BR>";
	
	// check server
	$url=get_server_root_url($sid,false);
	
	// parse url
	$urlparts=parse_url($url);
	
	// authentication
	$basicauth="";
	$urlauth="";
	if($sinfo["auth"]=="basic")
		$basicauth=$username.":".$password."@";
	if($sinfo["type"]=="nagiosxi")
		$urlauth="&username=".$username."&password=".md5($password);
	
	// construct new url
	$newurl=$urlparts["scheme"]."://".$basicauth.$urlparts["host"];
	if(array_key_exists("port",$urlparts))
		$newurl.=":".$urlparts["port"];
	$newurl.=$urlparts["path"]."/";

	if($sinfo["type"]=="nagiosxi"){
		$newurl.="backend/";
		$newurl.="?cmd=getticket";
		$newurl.=$urlauth;
		}

	
	//echo "FETCHING $newurl<BR><BR>";
	
	$args=array(
		"timeout" => 10,
		"return_info" => true,
		"method" => "post",
		);
	$urlres=load_url($newurl,$args);
	
	$http_code=$urlres['info']['http_code'];

	// first determine result by http code
	switch($http_code){
		case 0:
			$output="Server Timeout";
			$result=false;
			break;
		case 200:
			$output="OK ($http_code)";  //added http_code for clarity -MG 
			$result=true;
			break;
		case 401:
			$output="AUTH ERROR ($http_code)";
			$result=false;
			break;
		default:
			$result=false;
			$output.="AUTH UNKNOWN ($http_code)";
			break;
		}
		
	// check nagiosxi backend result
	if($sinfo["type"]=="nagiosxi" && $result==true){
		
		$output="AUTH BAD";
		$result=false;
		
		$body=$urlres['body'];
		//$output=$body;
		$xml=@simplexml_load_string($body);
		if($xml){
			$ticket=strval($xml);
			//$ticket=intval($xml[1]);
			//$output="TICKET=$ticket";
			// we got a ticket!
			if(strlen($ticket)>0){		//added strlen function.  ($ticket>0)  always returns false -MG 5/3/11
				$output="OK";
				$result=true;
				}
			}
		//print_r($xml);
		}
	
	//print_r($urlres);
	//echo "<BR><BR>";
	
	
	$res=$result;
	return $result;
	}
	
	
function get_server_data($sid,$username,$password,$type,&$arr){

	$arr=array(
		"timeout" => false,
		"http_code" => 0,
		"url" => "",
		"data" => "",
		);

	$sinfo=get_server_info($sid);
	
	//print_r($sinfo);
	//echo "<BR><BR>";
	
	// check server
	$url=get_server_root_url($sid,false);
	
	// parse url
	$urlparts=parse_url($url);
	
	// authentication
	$basicauth="";
	$urlauth="";
	if($sinfo["auth"]=="basic")
		$basicauth=$username.":".$password."@";
	else{
		if($sinfo["type"]=="nagiosxi")
			$urlauth="&username=".$username."&password=".md5($password);
		}
	
	// construct new url
	$newurl=$urlparts["scheme"]."://".$basicauth.$urlparts["host"];
	if(array_key_exists("port",$urlparts))
		$newurl.=":".$urlparts["port"];
	$newurl.=$urlparts["path"]."/";

	if($sinfo["type"]=="nagiosxi"){
		$newurl.="backend/";
		switch($type){
			case "hoststatus":
				$newurl.="?cmd=gethoststatus&brevity=1";
				break;
			case "servicestatus":
				$newurl.="?cmd=getservicestatus&combinedhost=true&brevity=1";
				break;
			case "programstatus":
				$newurl.="?cmd=getprogramstatus";
				break;
			default:
				$newurl.="?cmd=getticket";
				break;
			}
		$newurl.=$urlauth;
		}
	else if($sinfo["type"]=="nagioscore"){
		$newurl.="cgi-bin/";
		switch($type){
			case "tac":
				//$newurl.="tac-xml.cgi?fusion=1";
				$newurl.='tac.cgi?';
				break;
			default:
				$newurl.="?";
				break;
			}
		}
	/////////////////////////////////////////////////////////////////////////	
	//testing for V-Shell Integration	
	else if($sinfo['type']=='nagiosvshell') {
		$newurl.='?type=backend&mode=xml';
	}	
		
	// save url
	$arr["url"]=$newurl;
	
	//echo "FETCHING $newurl<BR><BR>";
	
	$args=array(
		"timeout" => 10,		//increasing timeout for servers with heavy load -MG 
		"return_info" => true,
		"method" => "post",
		);
	
	if($sinfo['type']=='nagiosvshell') $args['method'] = 'get';	
		
	$urlres=load_url($newurl,$args);
	
	$http_code=$urlres['info']['http_code'];
	
	// save httpd code
	$arr["http_code"]=$http_code;
	
	// did a timeout occur
	if($http_code==0)
		$arr["timeout"]=true;
	
	// save body
	$arr["body"]=$urlres['body'];
	
	return;
	}
	
	
function embed_nagiosxi_magic_login($sid){
	global $cfg;
	
	$enable_login=grab_array_var($cfg,'enable_auto_login',true);
	if($enable_login!=true)
		return "";

	$sinfo=get_server_info($sid);
	
	// get saved credentials
	$sc=get_option("server_credentials");
	if($sc==null)
		$sc=array();
	else
		$sc=unserialize($sc);
		
	$username=$sc[$sid]["username"];
	$password=$sc[$sid]["password"];
		
	$url=$sinfo["url"]."/backend/?cmd=getmagicpixel&username=".urlencode($username)."&password=".md5($password);
	
	$output="<img src='".$url."'>";
	
	return $output;
	}
	
	
////////////////////////////////////////////////////////////////////////
// SERVER FUNCTIONS
////////////////////////////////////////////////////////////////////////

// returns an array of information about all servers
function get_servers(){

	// get servers list
	$servers_serial=get_option("servers");
	if($servers_serial==null)
		$servers=array();
	else
		$servers=unserialize($servers_serial);
		
	return $servers;
	}
	
// returns an array with information about a specific server
function get_server_info($sid){

	$servers=get_servers();
	
	$sinfo=array();
	
	if(array_key_exists($sid,$servers))
		$sinfo=$servers[$sid];

	return $sinfo;
	}
	
// adds/updates a server
function update_server($sinfo,$sid=""){

	// get servers list
	$servers=get_servers();
	
	// add
	if($sid==""){
		$sid=random_string(6);
		$servers[$sid]=$sinfo;
		}
	// update
	else{
		$servers[$sid]=$sinfo;
		}
		
	// save new servers list
	set_option("servers",serialize($servers));
	
	// return new servers list
	return $servers;
	}
	
// deletes a server
function delete_server($sid){

	// get servers list
	$servers=get_servers();
	
	// array of servers
	if(is_array($sid)){
		foreach($sid as $server_id){
			unset($servers[$server_id]);
			}		
		}
	// single server
	else
		unset($servers[$sid]);
	
	// save new servers list
	set_option("servers",serialize($servers));
	
	// return new servers list
	return $servers;
	}
?>